"""
Copyright 2021 cpyberry
https://github.com/cpyberry/pybitfield

cpyberry
email: cpyberry222@gmail.com
github: https://github.com/cpyberry
"""


__title__ = "pybitfield"
__description__ = "A library that makes it easy to handle bitfields with python"
__copyright__ = "Copyright 2021 cpyberry"
__url__ = "https://github.com/cpyberry/pybitfield"
__license__ = "Apache-2.0 License"
__version__ = "1.0.1"
__author__ = "cpyberry"
__author_email__ = "cpyberry222@gmail.com"
__python_requires__ = ">=3.6"
