"""
Copyright 2021 cpyberry
https://github.com/cpyberry/pybitfield

cpyberry
email: cpyberry222@gmail.com
github: https://github.com/cpyberry
"""


from .bitfield import BitOrder, Bitfield

from .__info__ import (
	__title__,
	__description__,
	__copyright__,
	__url__,
	__license__,
	__version__,
	__author__,
	__author_email__,
	__python_requires__
)
